@@ -21,7 +21,7 @@ kdt_match
 import numpy as np
 from scipy import interpolate as interp
 
-from . import spectra, utils
+from . import spectra, utils, sift
 
 # Housekeeping for logging
 import logging
@@ -124,12 +124,17 @@ def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear'):
         array containing the phase aligned observations
 
     """
+    logger.info('STARTED: phase-align cycles')
 
     phase_edges, phase_bins = spectra.define_hist_bins(0, 2 * np.pi, npoints)
 
     if cycles is None:
         cycles = get_cycle_inds(ip)
 
+    logger.debug('aligning {0} cycles over {1} phase points with {2} interpolation'.format(cycles.max(),
+                                                                                           npoints,
+                                                                                           interp_kind))
+
     ncycles = cycles.max()
     avg = np.zeros((npoints, ncycles))
     for ii in range(1, ncycles + 1):
@@ -142,6 +147,7 @@ def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear'):
 
         avg[:, ii - 1] = f(phase_bins)
 
+    logger.info('COMPLETED: phase-align cycles')
     return avg
 
 
@@ -180,8 +186,8 @@ def get_cycle_inds(phase, return_good=True, mask=None,
     -----
     Good cycles are those with
     1 : A strictly positively increasing phase
-    2 : A phase starting within phase_step of zero (ie 0 < x < phase_step)
-    3 : A phase ending within phase_step of 2pi (is 2pi-phase_step < x < 2pi)
+    2 : A phase starting within phase_step of zero (ie 0 < x < phase_edge)
+    3 : A phase ending within phase_step of 2pi (is 2pi-phase_edge < x < 2pi)
     4 : A set of 4 unique control points
             (ascending zero, peak, descending zero & trough)
 
@@ -200,6 +206,12 @@ def get_cycle_inds(phase, return_good=True, mask=None,
 
     """
 
+    logger.info('STARTED: get cycle indices')
+    logger.debug('computing on {0} samples over {1} IMFs '.format(phase.shape[0],
+                                                                  phase.shape[1]))
+    if mask is not None:
+        logger.debug('{0} ({1}%) samples masked out'.format(mask.sum(), np.round(100*(mask.sum()/phase.shape[0]), 2)))
+
     if phase.max() > 2 * np.pi:
         print('Wrapping phase')
         phase = utils.wrap_phase(phase)
@@ -255,9 +267,9 @@ def get_cycle_inds(phase, return_good=True, mask=None,
                     try:
                         cycle = imf[inds[jj]:inds[jj + 1]]
                         # Should extend this to cope with multiple peaks etc
-                        ctrl = (0, utils.find_extrema(cycle)[0][0],
+                        ctrl = (0, sift.find_extrema(cycle)[0][0],
                                 np.where(np.gradient(np.sign(cycle)) == -1)[0][0],
-                                utils.find_extrema(-cycle)[0][0],
+                                sift.find_extrema(-cycle)[0][0],
                                 len(cycle))
                         if len(ctrl) == 5 and np.all(np.sign(np.diff(ctrl))):
                             cycle_checks[3] = True
@@ -277,10 +289,13 @@ def get_cycle_inds(phase, return_good=True, mask=None,
                 cycles[inds[jj]:inds[jj + 1], ii] = count
                 count += 1
 
+        logger.info('found {0} cycles in IMF-{1}'.format(cycles[:, ii].max(), ii))
+
+    logger.info('COMPLETED: get cycle indices')
     return cycles
 
 
-def get_cycle_stat(cycles, values, mode='compressed', metric='mean'):
+def get_cycle_stat(cycles, values, mode='compressed', func=np.mean):
     """
     Compute the average of a set of observations for each cycle.
 
@@ -294,6 +309,10 @@ def get_cycle_stat(cycles, values, mode='compressed', metric='mean'):
          Flag to indicate whether to return a single value per cycle or the
          average values filled within a vector of the same size as values
          (Default value = 'compressed')
+    func : function
+        Function to call on the data in values for each cycle (Default
+        np.mean). This can be any function, built-in or user defined, that
+        processes a single vector of data returning a single value.
 
     Returns
     -------
@@ -302,25 +321,36 @@ def get_cycle_stat(cycles, values, mode='compressed', metric='mean'):
 
 
     """
+    if (cycles.ndim > 1) and (cycles.shape[1] > 1):
+        raise ValueError('Cycles for {0} IMFs passed in, \
+                          please input the cycles for a single IMF'.format(cycles.shape[1]))
 
-    # https://stackoverflow.com/a/39598529
-    unq, ids, count = np.unique(cycles, return_inverse=True, return_counts=True)
-    vals = np.bincount(ids, values)
+    logger.info('STARTED: get cycle stats')
+    logger.debug('computing stats for {0} cycles over {1} samples'.format(cycles.max(), cycles.shape[0]))
+    logger.debug('computing metric {0} and returning {1}-array'.format(func, mode))
 
-    if metric == 'mean':
-        vals = vals / count
-    elif metric != 'sum':
-        # We already have the sum so just check the argument is as expected
-        raise ValueError('Metric not recognise, please use either \'mean\' or \'sum\'')
+    if mode == 'compressed':
+        out = np.zeros((cycles.max() + 1, )) * np.nan
+    elif mode == 'full':
+        out = np.zeros_like(values) * np.nan
 
-    if mode == 'full':
-        ret = np.zeros_like(cycles, dtype=float)
-        ret.fill(np.nan)
-        for ii in range(1, cycles.max() + 1):
-            ret[cycles == ii] = vals[ii - 1]
-        vals = ret
+    for cind in range(1, cycles.max() + 1):
+        stat = func(values[cycles == cind])
 
-    return vals
+        if mode == 'compressed':
+            out[cind] = stat
+        elif mode == 'full':
+            out[cycles == cind] = stat
+
+    # Currently including the first value as the stat for 'non-cycles' in
+    # compressed mode for backwards compatibility with earlier work, might be
+    # confusing overall - should probably rethink this whether this makes any
+    # sense
+    if mode == 'compressed':
+        out[0] = func(values[cycles == 0])
+
+    logger.info('COMPLETED: get cycle stats')
+    return out
 
 
 def get_control_points(x, good_cycles):
@@ -349,9 +379,9 @@ def get_control_points(x, good_cycles):
 
         # Note! we're currently just taking the first peak or trough if there
         # are more than one. This is dumb.
-        ctrl.append((0, utils.find_extrema(cycle)[0][0],
+        ctrl.append((0, sift.find_extrema(cycle)[0][0],
                      np.where(np.gradient(np.sign(cycle)) == -1)[0][0],
-                     utils.find_extrema(-cycle)[0][0],
+                     sift.find_extrema(-cycle)[0][0],
                      len(cycle)))
 
     return np.array(ctrl)
